@@ -22,6 +22,7 @@ use {
     cryptographic_message_syntax::time_stamp_message_http,
     goblin::mach::{
         constants::{SEG_LINKEDIT, SEG_PAGEZERO, SEG_TEXT},
+        header::MH_EXECUTE,
         load_command::{
             CommandVariant, LinkeditDataCommand, LC_BUILD_VERSION, SIZEOF_LINKEDIT_DATA_COMMAND,
         },
@@ -41,6 +42,9 @@ pub trait AppleSignable {
     /// Determine the start and end offset of the executable segment of a binary.
     fn executable_segment_boundary(&self) -> Result<(u64, u64), AppleCodesignError>;
 
+    /// Whether this is an executable Mach-O file.
+    fn is_executable(&self) -> bool;
+
     /// The start offset of the code signature data within the __LINKEDIT segment.
     fn code_signature_linkedit_start_offset(&self) -> Option<u32>;
 
@@ -65,6 +69,9 @@ pub trait AppleSignable {
     /// Resolve the load command for the code signature.
     fn code_signature_load_command(&self) -> Option<LinkeditDataCommand>;
 
+    /// Attempt to locate embedded Info.plist data.
+    fn embedded_info_plist(&self) -> Result<Option<Vec<u8>>, AppleCodesignError>;
+
     /// Determines whether this crate is capable of signing a given Mach-O binary.
     ///
     /// Code in this crate is limited in the amount of Mach-O binary manipulation
@@ -107,6 +114,10 @@ impl<'a> AppleSignable for MachO<'a> {
         Ok((segment.fileoff, segment.fileoff + segment.data.len() as u64))
     }
 
+    fn is_executable(&self) -> bool {
+        self.header.filetype == MH_EXECUTE
+    }
+
     fn code_signature_linkedit_start_offset(&self) -> Option<u32> {
         let segment = self
             .segments
@@ -185,6 +196,22 @@ impl<'a> AppleSignable for MachO<'a> {
         })
     }
 
+    fn embedded_info_plist(&self) -> Result<Option<Vec<u8>>, AppleCodesignError> {
+        // Mach-O binaries can have the Info.plist data in an `__info_plist` section
+        // within the __TEXT segment.
+        for segment in &self.segments {
+            if matches!(segment.name(), Ok(SEG_TEXT)) {
+                for (section, data) in segment.sections()? {
+                    if matches!(section.name(), Ok("__info_plist")) {
+                        return Ok(Some(data.to_vec()));
+                    }
+                }
+            }
+        }
+
+        Ok(None)
+    }
+
     fn check_signing_capability(&self) -> Result<(), AppleCodesignError> {
         let last_segment = self
             .segments
@@ -257,14 +284,14 @@ impl<'a> AppleSignable for MachO<'a> {
         // Reserve room for the code digests, which are proportional to binary size.
         // We could avoid doing the actual digesting work here. But until people
         // complain, don't worry about it.
-        size += compute_code_hashes(self, *settings.digest_type(), None)?
+        size += compute_code_hashes(self, *settings.digest_type(), 4096)?
             .into_iter()
             .map(|x| x.len())
             .sum::<usize>();
 
         if let Some(digests) = settings.extra_digests(SettingsScope::Main) {
             for digest in digests {
-                size += compute_code_hashes(self, *digest, None)?
+                size += compute_code_hashes(self, *digest, 4096)?
                     .into_iter()
                     .map(|x| x.len())
                     .sum::<usize>();
@@ -314,19 +341,19 @@ impl<'a> AppleSignable for MachO<'a> {
 /// Iterate [MachO] instances from file data.
 pub fn iter_macho(
     macho_data: &[u8],
-) -> Result<impl Iterator<Item = MachO<'_>>, AppleCodesignError> {
+) -> Result<impl Iterator<Item = (MachO<'_>, &'_ [u8])>, AppleCodesignError> {
     let mach = Mach::parse(macho_data)?;
 
     let machos = match mach {
         Mach::Binary(macho) => {
-            vec![macho]
+            vec![(macho, macho_data)]
         }
         Mach::Fat(multiarch) => {
             let mut machos = vec![];
 
-            for index in 0..multiarch.narches {
+            for (index, arch) in multiarch.arches()?.into_iter().enumerate() {
                 let macho = multiarch.get(index)?;
-                machos.push(macho);
+                machos.push((macho, arch.slice(macho_data)));
             }
 
             machos
@@ -498,6 +525,26 @@ impl From<u32> for Platform {
     }
 }
 
+impl Platform {
+    /// Resolve SHA-256 digest/signatures support for a given platform type.
+    pub fn sha256_digest_support(&self) -> Result<semver::VersionReq, AppleCodesignError> {
+        let version = match self {
+            // macOS 10.11.4 introduced support for SHA-256.
+            Self::MacOs => ">=10.11.4",
+            // 11.0+ support SHA-256.
+            Self::IOs | Self::TvOs => ">=11.0.0",
+            // WatchOS always uses SHA-1 it appears.
+            Self::WatchOs => ">9999",
+            // Assume no platform needs SHA-1.
+            Self::Unknown(0) => ">9999",
+            // Assume everything else is new and supports SHA-256.
+            _ => "*",
+        };
+
+        Ok(semver::VersionReq::parse(version)?)
+    }
+}
+
 /// Targeting settings for a Mach-O binary.
 pub struct MachoTarget {
     /// The OS/platform being targeted.
