@@ -8,23 +8,19 @@
 
 use {
     crate::{
-        code_directory::{CodeDirectoryBlob, CodeSignatureFlags},
+        code_directory::{CodeDirectoryBlob, CodeSignatureFlags, ExecutableSegmentFlags},
         code_hash::compute_code_hashes,
         code_requirement::{CodeRequirementExpression, CodeRequirements, RequirementType},
         embedded_signature::{
-            BlobData, CodeSigningSlot, Digest, EmbeddedSignature, EntitlementsBlob,
-            EntitlementsDerBlob, RequirementSetBlob,
+            BlobData, CodeSigningSlot, Digest, EntitlementsBlob, EntitlementsDerBlob,
+            RequirementSetBlob,
         },
         embedded_signature_builder::EmbeddedSignatureBuilder,
         entitlements::plist_to_executable_segment_flags,
         error::AppleCodesignError,
-        macho::{
-            find_macho_targeting, iter_macho, parse_version_nibbles,
-            semver_to_macho_target_version, AppleSignable,
-        },
+        macho::{find_macho_targeting, iter_macho, semver_to_macho_target_version, AppleSignable},
         policy::derive_designated_requirements,
         signing_settings::{DesignatedRequirementMode, SettingsScope, SigningSettings},
-        ExecutableSegmentFlags,
     },
     goblin::mach::{
         constants::{SEG_LINKEDIT, SEG_PAGEZERO},
@@ -190,6 +186,9 @@ fn create_macho_with_signature(
                 // overlapping with already written data. So only write segment
                 // data new to the writer.
                 if segment.fileoff < cursor.position() {
+                    if segment.data.is_empty() {
+                        continue;
+                    }
                     let remaining =
                         &segment.data[cursor.position() as usize..segment.filesize as usize];
                     cursor.write_all(remaining)?;
@@ -238,7 +237,11 @@ impl<'data> MachOSigner<'data> {
     /// The data will be parsed as a Mach-O binary (either single arch or fat/universal)
     /// and validated that we are capable of signing it.
     pub fn new(macho_data: &'data [u8]) -> Result<Self, AppleCodesignError> {
-        let machos = iter_macho(macho_data)?.collect::<Vec<_>>();
+        let machos = iter_macho(macho_data)?
+            .collect::<Vec<_>>()
+            .into_iter()
+            .map(|(x, _)| x)
+            .collect::<Vec<_>>();
 
         Ok(Self { macho_data, machos })
     }
@@ -277,12 +280,8 @@ impl<'data> MachOSigner<'data> {
                 // A nice side-effect of this is that it catches bugs if we write malformed Mach-O!
                 let intermediate_macho = MachO::parse(&intermediate_macho_data, 0)?;
 
-                let mut signature_data = self.create_superblob(
-                    &settings,
-                    self.macho_data(index),
-                    &intermediate_macho,
-                    original_macho.code_signature()?.as_ref(),
-                )?;
+                let mut signature_data =
+                    self.create_superblob(&settings, self.macho_data(index), &intermediate_macho)?;
                 info!("total signature size: {} bytes", signature_data.len());
 
                 // The Mach-O writer adjusts load commands based on the signature length. So pad
@@ -347,16 +346,14 @@ impl<'data> MachOSigner<'data> {
         settings: &SigningSettings,
         macho_data: &[u8],
         macho: &MachO,
-        previous_signature: Option<&EmbeddedSignature>,
     ) -> Result<Vec<u8>, AppleCodesignError> {
         let mut builder = EmbeddedSignatureBuilder::default();
 
-        for (slot, blob) in self.create_special_blobs(settings, previous_signature)? {
+        for (slot, blob) in self.create_special_blobs(settings, macho.is_executable())? {
             builder.add_blob(slot, blob)?;
         }
 
-        let code_directory =
-            self.create_code_directory(settings, macho_data, macho, previous_signature)?;
+        let code_directory = self.create_code_directory(settings, macho_data, macho)?;
         info!("code directory version: {}", code_directory.version);
 
         builder.add_code_directory(CodeSigningSlot::CodeDirectory, code_directory)?;
@@ -372,12 +369,7 @@ impl<'data> MachOSigner<'data> {
                     "adding alternative code directory using digest {:?}",
                     digest_type
                 );
-                let cd = self.create_code_directory(
-                    &alt_settings,
-                    macho_data,
-                    macho,
-                    previous_signature,
-                )?;
+                let cd = self.create_code_directory(&alt_settings, macho_data, macho)?;
 
                 builder.add_alternative_code_directory(cd.into())?;
             }
@@ -395,30 +387,6 @@ impl<'data> MachOSigner<'data> {
         builder.create_superblob()
     }
 
-    /// Attempt to resolve the binary identifier to use.
-    ///
-    /// If signing settings have defined one, use it. Otherwise use the last
-    /// identifier on the binary, if present. Otherwise error.
-    fn get_binary_identifier(
-        &self,
-        settings: &SigningSettings,
-        previous_signature: Option<&EmbeddedSignature>,
-    ) -> Result<String, AppleCodesignError> {
-        let previous_cd =
-            previous_signature.and_then(|signature| signature.code_directory().unwrap_or(None));
-
-        match settings.binary_identifier(SettingsScope::Main) {
-            Some(ident) => Ok(ident.to_string()),
-            None => {
-                if let Some(previous_cd) = &previous_cd {
-                    Ok(previous_cd.ident.to_string())
-                } else {
-                    Err(AppleCodesignError::NoIdentifier)
-                }
-            }
-        }
-    }
-
     /// Create the `CodeDirectory` for the current configuration.
     ///
     /// This takes an explicit Mach-O to operate on due to a circular dependency
@@ -429,7 +397,6 @@ impl<'data> MachOSigner<'data> {
         settings: &SigningSettings,
         macho_data: &[u8],
         macho: &MachO,
-        previous_signature: Option<&EmbeddedSignature>,
     ) -> Result<CodeDirectoryBlob<'static>, AppleCodesignError> {
         // TODO support defining or filling in proper values for fields with
         // static values.
@@ -443,31 +410,14 @@ impl<'data> MachOSigner<'data> {
             );
         }
 
-        let previous_cd = previous_signature.and_then(|signature| {
-            signature
-                .code_directory_for_digest(*settings.digest_type())
-                .unwrap_or(None)
-        });
-
         let mut flags = CodeSignatureFlags::empty();
 
-        match settings.code_signature_flags(SettingsScope::Main) {
-            Some(additional) => {
-                info!(
-                    "adding code signature flags from signing settings: {:?}",
-                    additional
-                );
-                flags |= additional
-            }
-            None => {
-                if let Some(previous_cd) = &previous_cd {
-                    info!(
-                        "copying code signature flags from previous code directory: {:?}",
-                        previous_cd.flags
-                    );
-                    flags |= previous_cd.flags;
-                }
-            }
+        if let Some(additional) = settings.code_signature_flags(SettingsScope::Main) {
+            info!(
+                "adding code signature flags from signing settings: {:?}",
+                additional
+            );
+            flags |= additional;
         }
 
         // The adhoc flag is set when there is no CMS signature.
@@ -499,49 +449,28 @@ impl<'data> MachOSigner<'data> {
         let (exec_seg_base, exec_seg_limit) = macho.executable_segment_boundary()?;
         let (exec_seg_base, exec_seg_limit) = (Some(exec_seg_base), Some(exec_seg_limit));
 
-        let mut exec_seg_flags = None;
+        // Executable segment flags are wonky.
+        //
+        // Foremost, these flags are only present if the Mach-O binary is an executable. So not
+        // matter what the settings say, we don't set these flags unless the Mach-O file type
+        // is proper.
+        //
+        // Executable segment flags are also derived from an associated entitlements plist.
+        let exec_seg_flags = if macho.is_executable() {
+            if let Some(entitlements) = settings.entitlements_plist(SettingsScope::Main) {
+                let flags = plist_to_executable_segment_flags(entitlements);
 
-        match settings.executable_segment_flags(SettingsScope::Main) {
-            Some(flags) => {
-                info!(
-                    "using executable segment flags from signing settings ({:?})",
-                    flags
-                );
-                exec_seg_flags = Some(flags);
-            }
-            None => {
-                if let Some(previous_cd) = &previous_cd {
-                    if let Some(flags) = previous_cd.exec_seg_flags {
-                        info!(
-                            "using executable segment flags from previous code directory ({:?})",
-                            flags
-                        );
-                        exec_seg_flags = Some(flags);
-                    }
+                if !flags.is_empty() {
+                    info!("entitlements imply executable segment flags: {:?}", flags);
                 }
+
+                Some(flags | ExecutableSegmentFlags::MAIN_BINARY)
+            } else {
+                Some(ExecutableSegmentFlags::MAIN_BINARY)
             }
-        }
-
-        // Entitlements can influence the executable segment flags. So make sure
-        // flags derived from entitlements are always set.
-        if let Some(entitlements) = settings.entitlements_plist(SettingsScope::Main) {
-            let implied_flags = plist_to_executable_segment_flags(entitlements);
-
-            if !implied_flags.is_empty() {
-                info!(
-                    "entitlements imply executable segment flags: {:?}",
-                    implied_flags
-                );
-
-                exec_seg_flags = Some(
-                    exec_seg_flags.unwrap_or_else(ExecutableSegmentFlags::empty) | implied_flags,
-                );
-            }
-        }
-
-        if let Some(flags) = exec_seg_flags {
-            info!("final executable segment flags: {:?}", flags);
-        }
+        } else {
+            None
+        };
 
         // The runtime version is the SDK version from the targeting loader commands. Same
         // u32 with nibbles encoding the version.
@@ -549,20 +478,6 @@ impl<'data> MachOSigner<'data> {
         // If the runtime code signature flag is set, we also need to set the runtime version
         // or else the activation of the hardened runtime is incomplete.
 
-        // First default to the existing runtime version, if present.
-        let runtime = match &previous_cd {
-            Some(previous_cd) => {
-                if let Some(version) = previous_cd.runtime {
-                    info!(
-                        "copying hardened runtime version {} from previous code directory",
-                        parse_version_nibbles(version)
-                    );
-                }
-                previous_cd.runtime
-            }
-            None => None,
-        };
-
         // If the settings defines a runtime version override, use it.
         let runtime = match settings.runtime_version(SettingsScope::Main) {
             Some(version) => {
@@ -572,7 +487,7 @@ impl<'data> MachOSigner<'data> {
                 );
                 Some(semver_to_macho_target_version(version))
             }
-            None => runtime,
+            None => None,
         };
 
         // If we still don't have a runtime but need one, derive from the target SDK.
@@ -591,74 +506,44 @@ impl<'data> MachOSigner<'data> {
             runtime
         };
 
-        let code_hashes =
-            compute_code_hashes(macho, *settings.digest_type(), Some(page_size as usize))?
-                .into_iter()
-                .map(|v| Digest { data: v.into() })
-                .collect::<Vec<_>>();
+        let code_hashes = compute_code_hashes(macho, *settings.digest_type(), page_size as _)?
+            .into_iter()
+            .map(|v| Digest { data: v.into() })
+            .collect::<Vec<_>>();
 
         let mut special_hashes = HashMap::new();
 
         // There is no corresponding blob for the info plist data since it is provided
         // externally to the embedded signature.
-        match settings.info_plist_data(SettingsScope::Main) {
-            Some(data) => {
-                special_hashes.insert(
-                    CodeSigningSlot::Info,
-                    Digest {
-                        data: settings.digest_type().digest_data(data)?.into(),
-                    },
-                );
-            }
-            None => {
-                if let Some(previous_cd) = &previous_cd {
-                    if let Some(digest) = previous_cd.slot_digest(CodeSigningSlot::Info) {
-                        if !digest.is_null() {
-                            special_hashes.insert(CodeSigningSlot::Info, digest.to_owned());
-                        }
-                    }
-                }
-            }
+        if let Some(data) = settings.info_plist_data(SettingsScope::Main) {
+            special_hashes.insert(
+                CodeSigningSlot::Info,
+                Digest {
+                    data: settings.digest_type().digest_data(data)?.into(),
+                },
+            );
         }
 
         // There is no corresponding blob for resources data since it is provided
         // externally to the embedded signature.
-        match settings.code_resources_data(SettingsScope::Main) {
-            Some(data) => {
-                special_hashes.insert(
-                    CodeSigningSlot::ResourceDir,
-                    Digest {
-                        data: settings.digest_type().digest_data(data)?.into(),
-                    }
-                    .to_owned(),
-                );
-            }
-            None => {
-                if let Some(previous_cd) = &previous_cd {
-                    if let Some(digest) = previous_cd.slot_digest(CodeSigningSlot::ResourceDir) {
-                        if !digest.is_null() {
-                            special_hashes.insert(CodeSigningSlot::ResourceDir, digest.to_owned());
-                        }
-                    }
+        if let Some(data) = settings.code_resources_data(SettingsScope::Main) {
+            special_hashes.insert(
+                CodeSigningSlot::ResourceDir,
+                Digest {
+                    data: settings.digest_type().digest_data(data)?.into(),
                 }
-            }
+                .to_owned(),
+            );
         }
 
-        let ident = Cow::Owned(self.get_binary_identifier(settings, previous_signature)?);
+        let ident = Cow::Owned(
+            settings
+                .binary_identifier(SettingsScope::Main)
+                .ok_or(AppleCodesignError::NoIdentifier)?
+                .to_string(),
+        );
 
-        let team_name = match settings.team_id() {
-            Some(team_name) => Some(Cow::Owned(team_name.to_string())),
-            None => {
-                if let Some(previous_cd) = &previous_cd {
-                    previous_cd
-                        .team_name
-                        .as_ref()
-                        .map(|name| Cow::Owned(name.clone().into_owned()))
-                } else {
-                    None
-                }
-            }
-        };
+        let team_name = settings.team_id().map(|x| Cow::Owned(x.to_string()));
 
         let mut cd = CodeDirectoryBlob {
             flags,
@@ -698,7 +583,7 @@ impl<'data> MachOSigner<'data> {
     pub fn create_special_blobs(
         &self,
         settings: &SigningSettings,
-        previous_signature: Option<&EmbeddedSignature>,
+        is_executable: bool,
     ) -> Result<Vec<(CodeSigningSlot, BlobData<'static>)>, AppleCodesignError> {
         let mut res = Vec::new();
 
@@ -710,8 +595,12 @@ impl<'data> MachOSigner<'data> {
                 // derive appropriate designated requirements.
                 if let Some((_, cert)) = settings.signing_key() {
                     info!("attempting to derive code requirements from signing certificate");
-                    let identifier =
-                        Some(self.get_binary_identifier(settings, previous_signature)?);
+                    let identifier = Some(
+                        settings
+                            .binary_identifier(SettingsScope::Main)
+                            .ok_or(AppleCodesignError::NoIdentifier)?
+                            .to_string(),
+                    );
 
                     if let Some(expr) = derive_designated_requirements(cert, identifier)? {
                         requirements.push(expr);
@@ -744,14 +633,16 @@ impl<'data> MachOSigner<'data> {
 
         // The DER encoded entitlements weren't always present in the signature. The feature
         // appears to have been introduced in macOS 10.14 and is the default behavior as of
-        // macOS 12 "when signing for all platforms." Since `codesign` appears to always add
-        // this blob when entitlements are present, we mimic the behavior. But there may be
-        // scenarios where we want to omit this.
-        if let Some(value) = settings.entitlements_plist(SettingsScope::Main) {
-            info!("adding entitlements DER");
-            let blob = EntitlementsDerBlob::from_plist(value)?;
+        // macOS 12 "when signing for all platforms." `codesign` appears to add the DER
+        // representation whenever entitlements are present, but only if the current binary is
+        // an executable (.filetype == MH_EXECUTE).
+        if is_executable {
+            if let Some(value) = settings.entitlements_plist(SettingsScope::Main) {
+                info!("adding entitlements DER");
+                let blob = EntitlementsDerBlob::from_plist(value)?;
 
-            res.push((CodeSigningSlot::EntitlementsDer, blob.into()));
+                res.push((CodeSigningSlot::EntitlementsDer, blob.into()));
+            }
         }
 
         Ok(res)
