@@ -61,7 +61,7 @@ use {
             create_self_signed_code_signing_certificate, parse_pfx_data, AppleCertificate,
             CertificateProfile,
         },
-        code_directory::{CodeDirectoryBlob, CodeSignatureFlags, ExecutableSegmentFlags},
+        code_directory::{CodeDirectoryBlob, CodeSignatureFlags},
         code_hash::compute_code_hashes,
         code_requirement::CodeRequirements,
         embedded_signature::{Blob, CodeSigningSlot, DigestType, RequirementSetBlob},
@@ -299,7 +299,6 @@ entities:
 * --code-resources-path
 * --code-signature-flags
 * --entitlements-xml-path
-* --executable-segment-flags
 * --info-plist-path
 
 Scoped settings take the form <value> or <scope>:<value>. If the 2nd form
@@ -354,6 +353,7 @@ settings ignored include (but may not be limited to):
 * --code-resources-path. The code resources data will be computed automatically
   as part of signing the bundle.
 * --info-plist-path. The `Info.plist` from the bundle will be used instead.
+* --digest and --extra-digest
 
 # Designated Code Requirements
 
@@ -810,11 +810,11 @@ fn command_compute_code_hashes(args: &ArgMatches) -> Result<(), AppleCodesignErr
     let index = args.value_of("universal_index").unwrap();
     let index = usize::from_str(index).map_err(|_| AppleCodesignError::CliBadArgument)?;
     let hash_type = DigestType::try_from(args.value_of("hash").unwrap())?;
-    let page_size = if let Some(page_size) = args.value_of("page_size") {
-        Some(usize::from_str(page_size).map_err(|_| AppleCodesignError::CliBadArgument)?)
-    } else {
-        None
-    };
+    let page_size = usize::from_str(
+        args.value_of("page_size")
+            .expect("page_size should have default value"),
+    )
+    .map_err(|_| AppleCodesignError::CliBadArgument)?;
 
     let data = std::fs::read(path)?;
     let macho = get_macho_from_data(&data, index)?;
@@ -1806,15 +1806,6 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
         }
     }
 
-    if let Some(values) = args.values_of("executable_segment_flags_set") {
-        for value in values {
-            let (scope, value) = parse_scoped_value(value)?;
-
-            let flags = ExecutableSegmentFlags::from_str(value)?;
-            settings.set_executable_segment_flags(scope, flags);
-        }
-    }
-
     if let Some(values) = args.values_of("runtime_version") {
         for value in values {
             let (scope, value) = parse_scoped_value(value)?;
@@ -2086,6 +2077,7 @@ fn main_impl() -> Result<(), AppleCodesignError> {
                 Arg::new("page_size")
                     .long("page-size")
                     .takes_value(true)
+                    .default_value("4096")
                     .help("Chunk size to digest over"),
             )
             .arg(
@@ -2427,15 +2419,6 @@ fn main_impl() -> Result<(), AppleCodesignError> {
                         .number_of_values(1)
                         .help("Path to a plist file containing entitlements"),
                 )
-                .arg(
-                    Arg::new("executable_segment_flags_set")
-                        .long("executable-segment-flags")
-                        .takes_value(true)
-                        .multiple_occurrences(true)
-                        .multiple_values(true)
-                        .number_of_values(1)
-                        .help("Executable segment flags to set")
-                )
                 .arg(
                     Arg::new("runtime_version")
                         .long("runtime-version")
