@@ -17,13 +17,18 @@ use {
     crate::{
         app_metadata::{Asset, DataFile, Package, SoftwareAssets},
         app_store_connect::{AppStoreConnectClient, ConnectToken, DevIdPlusInfoResponse},
+        dmg::DmgReader,
+        reader::PathType,
         AppleCodesignError,
     },
     apple_bundles::DirectoryBundle,
+    apple_flat_package::PkgReader,
     log::{error, info, warn},
     md5::Digest,
     std::{
-        io::{BufRead, Write},
+        fmt::Debug,
+        fs::File,
+        io::{BufRead, Cursor, Read, Seek, SeekFrom, Write},
         path::{Path, PathBuf},
         time::Duration,
     },
@@ -204,6 +209,25 @@ impl TransporterUploadCommand {
     }
 }
 
+fn digest_md5<R: Read>(reader: &mut R) -> Result<(u64, Vec<u8>), AppleCodesignError> {
+    let mut hasher = md5::Md5::new();
+    let mut size = 0;
+
+    loop {
+        let mut buffer = [0u8; 16384];
+        let count = reader.read(&mut buffer)?;
+
+        size += count as u64;
+        hasher.update(&buffer[0..count]);
+
+        if count < buffer.len() {
+            break;
+        }
+    }
+
+    Ok((size, hasher.finalize().to_vec()))
+}
+
 /// Produce zip file data from a [DirectoryBundle].
 ///
 /// The built zip file will contain all the files from the bundle under a directory
@@ -220,18 +244,32 @@ pub fn bundle_to_zip(bundle: &DirectoryBundle) -> Result<Vec<u8>, AppleCodesignE
             .as_file_entry()
             .map_err(AppleCodesignError::DirectoryBundle)?;
 
-        let options =
-            zip::write::FileOptions::default().unix_permissions(if entry.is_executable() {
-                0o0755
-            } else {
-                0o0644
-            });
+        let permissions = if entry.is_executable() {
+            0o0755
+        } else {
+            0o0644
+        };
+
+        let permissions = if entry.link_target().is_some() {
+            // S_IFLINK.
+            permissions | 0xa00
+        } else {
+            permissions
+        };
+
+        let options = zip::write::FileOptions::default().unix_permissions(permissions);
 
         zf.start_file(
             format!("{}/{}", bundle.name(), file.relative_path().display()),
             options,
         )?;
-        zf.write_all(&entry.resolve_content()?)?;
+
+        if let Some(target) = entry.link_target() {
+            // For symlinks the target is the content.
+            zf.write_all(target.to_string_lossy().as_bytes())?;
+        } else {
+            zf.write_all(&entry.resolve_content()?)?;
+        }
     }
 
     let writer = zf.finish()?;
@@ -288,9 +326,7 @@ pub fn write_bundle_to_app_store_package(
 
     // MD5 is always used for the digest.
     let checksum_type = "md5".to_string();
-    let mut h = md5::Md5::new();
-    h.update(&bundle_zip);
-    let checksum_digest = hex::encode(h.finalize());
+    let checksum_digest = hex::encode(digest_md5(&mut Cursor::new(&bundle_zip))?.1);
 
     let file_name = format!("{}.zip", bundle.name());
 
@@ -327,6 +363,158 @@ pub fn write_bundle_to_app_store_package(
     Ok(())
 }
 
+/// Write a DMG to an `.itmsp` directory.
+pub fn write_dmg_to_app_store_package(
+    source_path: &Path,
+    dmg: &DmgReader,
+    dest_dir: &Path,
+) -> Result<(), AppleCodesignError> {
+    let signature = dmg
+        .embedded_signature()?
+        .ok_or(AppleCodesignError::DmgNotarizeNoSignature)?;
+    let cd = signature
+        .code_directory()?
+        .ok_or(AppleCodesignError::DmgNotarizeNoSignature)?;
+
+    let primary_bundle_identifier = cd.ident.to_string();
+    info!("primary bundle identifier: {}", primary_bundle_identifier);
+
+    let app_platform = "osx".to_string();
+    let asset_type = "developer-id-package".to_string();
+    let file_name = source_path
+        .file_name()
+        .map(|x| x.to_string_lossy().to_string())
+        .unwrap_or_else(|| "image.dmg".to_string());
+    let checksum_type = "md5".to_string();
+    let (size, checksum_digest) = digest_md5(&mut File::open(source_path)?)?;
+    let checksum_digest = hex::encode(checksum_digest);
+
+    let package = Package {
+        software_assets: SoftwareAssets {
+            app_platform,
+            device_id: None,
+            primary_bundle_identifier,
+            asset: Asset {
+                typ: asset_type,
+                data_files: vec![DataFile {
+                    file_name: file_name.clone(),
+                    checksum_type,
+                    checksum_digest,
+                    size,
+                }],
+            },
+        },
+    };
+
+    let metadata_xml = package
+        .to_xml()
+        .map_err(AppleCodesignError::AppMetadataXml)?;
+
+    let dmg_path = dest_dir.join(&file_name);
+    info!("writing {}", dmg_path.display());
+    std::fs::copy(source_path, &dmg_path)?;
+
+    let metadata_path = dest_dir.join("metadata.xml");
+    info!("writing {}", metadata_path.display());
+    std::fs::write(&metadata_path, &metadata_xml)?;
+
+    Ok(())
+}
+
+/// Write a flat package (usually a `.pkg` file) to an `.itmsp` directory.
+pub fn write_flat_package_to_app_store_package<F: Read + Seek + Debug>(
+    mut pkg: PkgReader<F>,
+    dest_dir: &Path,
+) -> Result<(), AppleCodesignError> {
+    let primary_bundle_identifier = if let Some(distribution) = pkg.distribution()? {
+        warn!("notarizing a product installer");
+
+        // We need to extract the primary bundle identifier from the `Distribution` XML.
+        // We should probably honor the settings in the XML (this logic would belong in
+        // the apple-flat-package crate). But for now we just pick the first bundle ID
+        // we see.
+        let id = if let Some(id) = distribution.pkg_ref.iter().find_map(|x| {
+            if let Some(bv) = &x.bundle_version {
+                bv.bundle.get(0).map(|bundle| bundle.id.clone())
+            } else {
+                None
+            }
+        }) {
+            id
+        } else {
+            error!("unable to find bundle identifier in flat package (please report this bug)");
+            return Err(AppleCodesignError::NotarizeFlatPackageParse);
+        };
+
+        id
+    } else if let Some(_component) = pkg.root_component()? {
+        warn!("notarizing a component installer");
+
+        error!("support for notarizing a component installer is not yet implemented");
+        return Err(AppleCodesignError::NotarizeFlatPackageParse);
+    } else {
+        error!("do not know how to extract bundle identifier from package installer");
+        error!("please report this bug");
+
+        return Err(AppleCodesignError::NotarizeFlatPackageParse);
+    };
+
+    warn!(
+        "resolved primary bundle identifier to {}",
+        primary_bundle_identifier
+    );
+
+    // Are flat packages supported on other platforms?
+    let app_platform = "osx".to_string();
+    let asset_type = "developer-id-package".to_string();
+
+    // This doesn't appear to matter?
+    let file_name = "installer.pkg".to_string();
+
+    // In order to compute the digest we'll need access to the raw reader.
+    let mut fh = pkg.into_inner().into_inner();
+    fh.seek(SeekFrom::Start(0))?;
+
+    let checksum_type = "md5".to_string();
+    let (size, checksum_digest) = digest_md5(&mut fh)?;
+    let checksum_digest = hex::encode(checksum_digest);
+
+    let package = Package {
+        software_assets: SoftwareAssets {
+            app_platform,
+            device_id: None,
+            primary_bundle_identifier,
+            asset: Asset {
+                typ: asset_type,
+                data_files: vec![DataFile {
+                    file_name: file_name.clone(),
+                    checksum_type,
+                    checksum_digest,
+                    size,
+                }],
+            },
+        },
+    };
+
+    let metadata_xml = package
+        .to_xml()
+        .map_err(AppleCodesignError::AppMetadataXml)?;
+
+    let pkg_path = dest_dir.join(&file_name);
+    info!("writing {}", pkg_path.display());
+    fh.seek(SeekFrom::Start(0))?;
+    {
+        let mut ofh = std::fs::File::create(&pkg_path)?;
+        std::io::copy(&mut fh, &mut ofh)?;
+    }
+
+    let metadata_path = dest_dir.join("metadata.xml");
+    info!("writing {}", metadata_path.display());
+    std::fs::write(&metadata_path, &metadata_xml)?;
+
+    Ok(())
+}
+
 fn upload_id_from_json_str(s: &str) -> Result<Option<String>, AppleCodesignError> {
     let value = serde_json::from_str::<serde_json::Value>(s)?;
 
@@ -345,6 +533,21 @@ fn upload_id_from_json_str(s: &str) -> Result<Option<String>, AppleCodesignError
     }
 }
 
+fn create_itmsp_temp_dir() -> Result<(tempfile::TempDir, PathBuf), AppleCodesignError> {
+    let temp_dir = tempfile::Builder::new()
+        .prefix("apple-codesign-")
+        .tempdir()?;
+
+    let id = uuid::Uuid::new_v4().to_string();
+
+    let itmsp = format!("{}.itmsp", id);
+
+    let dest_dir = temp_dir.path().join(&itmsp);
+    std::fs::create_dir_all(&dest_dir)?;
+
+    Ok((temp_dir, dest_dir))
+}
+
 /// Represents the result of a notarization upload.
 pub enum NotarizationUpload {
     /// We performed the upload and only have the upload ID / UUID for it.
@@ -407,12 +610,25 @@ impl Notarizer {
         path: &Path,
         wait_limit: Option<Duration>,
     ) -> Result<NotarizationUpload, AppleCodesignError> {
-        if let Ok(bundle) = DirectoryBundle::new_from_path(path) {
-            self.notarize_bundle(&bundle, wait_limit)
-        } else {
-            Err(AppleCodesignError::NotarizeUnsupportedPath(
+        match PathType::from_path(path)? {
+            PathType::Bundle => {
+                let bundle = DirectoryBundle::new_from_path(path)
+                    .map_err(AppleCodesignError::DirectoryBundle)?;
+                self.notarize_bundle(&bundle, wait_limit)
+            }
+            PathType::Xar => {
+                let fh = File::options().read(true).write(true).open(path)?;
+                let pkg = PkgReader::new(fh)?;
+                self.notarize_flat_package(pkg, wait_limit)
+            }
+            PathType::Dmg => {
+                let mut fh = File::open(path)?;
+                let reader = DmgReader::new(&mut fh)?;
+                self.notarize_dmg(path, &reader, wait_limit)
+            }
+            PathType::MachO | PathType::Other => Err(AppleCodesignError::NotarizeUnsupportedPath(
                 path.to_path_buf(),
-            ))
+            )),
         }
     }
 
@@ -425,21 +641,49 @@ impl Notarizer {
         bundle: &DirectoryBundle,
         wait_limit: Option<Duration>,
     ) -> Result<NotarizationUpload, AppleCodesignError> {
-        let temp_dir = tempfile::Builder::new()
-            .prefix("apple-codesign-")
-            .tempdir()?;
-
-        let id = uuid::Uuid::new_v4().to_string();
-
-        let itmsp = format!("{}.itmsp", id);
-
-        let dest_dir = temp_dir.path().join(&itmsp);
-        std::fs::create_dir_all(&dest_dir)?;
+        let (_temp_dir, dest_dir) = create_itmsp_temp_dir()?;
         warn!("writing App Store Package to {}", dest_dir.display());
 
         write_bundle_to_app_store_package(bundle, &dest_dir)?;
 
-        let upload_id = self.upload_app_store_package(&dest_dir)?;
+        self.upload_directory_and_maybe_wait(&dest_dir, wait_limit)
+    }
+
+    /// Attempt to notarize a DMG file.
+    pub fn notarize_dmg(
+        &self,
+        dmg_path: &Path,
+        dmg: &DmgReader,
+        wait_limit: Option<Duration>,
+    ) -> Result<NotarizationUpload, AppleCodesignError> {
+        let (_temp_dir, dest_dir) = create_itmsp_temp_dir()?;
+        warn!("writing App Store Package to {}", dest_dir.display());
+
+        write_dmg_to_app_store_package(dmg_path, dmg, &dest_dir)?;
+
+        self.upload_directory_and_maybe_wait(&dest_dir, wait_limit)
+    }
+
+    /// Attempt to notarize a flat package (`.pkg`) installer.
+    pub fn notarize_flat_package<F: Read + Write + Seek + Sized + Debug>(
+        &self,
+        pkg: PkgReader<F>,
+        wait_limit: Option<Duration>,
+    ) -> Result<NotarizationUpload, AppleCodesignError> {
+        let (_temp_dir, dest_dir) = create_itmsp_temp_dir()?;
+        warn!("writing XAR to {}", dest_dir.display());
+
+        write_flat_package_to_app_store_package(pkg, &dest_dir)?;
+
+        self.upload_directory_and_maybe_wait(&dest_dir, wait_limit)
+    }
+
+    fn upload_directory_and_maybe_wait(
+        &self,
+        upload_dir: &Path,
+        wait_limit: Option<Duration>,
+    ) -> Result<NotarizationUpload, AppleCodesignError> {
+        let upload_id = self.upload_app_store_package(upload_dir)?;
 
         let status = if let Some(wait_limit) = wait_limit {
             self.wait_on_app_store_package_upload_and_fetch_log(&upload_id, wait_limit)?
@@ -594,6 +838,7 @@ impl Notarizer {
         response: &DevIdPlusInfoResponse,
     ) -> Result<String, AppleCodesignError> {
         if let Some(url) = &response.dev_id_plus.log_file_url {
+            info!("fetching log from {}", url);
             let client = crate::ticket_lookup::default_client()?;
 
             let response = client.get(url).send()?;
