@@ -7,11 +7,11 @@
 use {
     crate::{
         certificate::AppleCertificate,
-        code_directory::{CodeSignatureFlags, ExecutableSegmentFlags},
+        code_directory::CodeSignatureFlags,
         code_requirement::CodeRequirementExpression,
         embedded_signature::{Blob, DigestType, RequirementBlob},
         error::AppleCodesignError,
-        macho::{iter_macho, AppleSignable},
+        macho::{find_macho_targeting, iter_macho, parse_version_nibbles, AppleSignable},
     },
     glob::Pattern,
     goblin::mach::cputype::{
@@ -260,18 +260,17 @@ pub struct SigningSettings<'key> {
     signing_key: Option<(&'key dyn Sign, CapturedX509Certificate)>,
     certificates: Vec<CapturedX509Certificate>,
     time_stamp_url: Option<Url>,
-    team_id: Option<String>,
     digest_type: DigestType,
     path_exclusion_patterns: Vec<Pattern>,
 
     // Scope-specific settings.
     // These are BTreeMap so when we filter the keys, keys with higher precedence come
     // last and last write wins.
+    team_id: BTreeMap<SettingsScope, String>,
     identifiers: BTreeMap<SettingsScope, String>,
     entitlements: BTreeMap<SettingsScope, plist::Value>,
     designated_requirement: BTreeMap<SettingsScope, DesignatedRequirementMode>,
     code_signature_flags: BTreeMap<SettingsScope, CodeSignatureFlags>,
-    executable_segment_flags: BTreeMap<SettingsScope, ExecutableSegmentFlags>,
     runtime_version: BTreeMap<SettingsScope, semver::Version>,
     info_plist_data: BTreeMap<SettingsScope, Vec<u8>>,
     code_resources_data: BTreeMap<SettingsScope, Vec<u8>>,
@@ -398,12 +397,12 @@ impl<'key> SigningSettings<'key> {
 
     /// Obtain the team identifier for signed binaries.
     pub fn team_id(&self) -> Option<&str> {
-        self.team_id.as_deref()
+        self.team_id.get(&SettingsScope::Main).map(|x| x.as_str())
     }
 
     /// Set the team identifier for signed binaries.
     pub fn set_team_id(&mut self, value: impl ToString) {
-        self.team_id = Some(value.to_string());
+        self.team_id.insert(SettingsScope::Main, value.to_string());
     }
 
     /// Attempt to set the team ID from the signing certificate.
@@ -426,7 +425,11 @@ impl<'key> SigningSettings<'key> {
         if let Some((_, cert)) = &self.signing_key {
             if let Some(team_id) = cert.apple_team_id() {
                 self.set_team_id(team_id);
-                Some(self.team_id.as_ref().expect("we just set a team id"))
+                Some(
+                    self.team_id
+                        .get(&SettingsScope::Main)
+                        .expect("we just set a team id"),
+                )
             } else {
                 None
             }
@@ -503,13 +506,55 @@ impl<'key> SigningSettings<'key> {
         &mut self,
         macho_data: &[u8],
     ) -> Result<(), AppleCodesignError> {
-        for (index, macho) in iter_macho(macho_data)?.enumerate() {
+        info!("inferring default signing settings from Mach-O binary");
+
+        for (index, (macho, macho_data)) in iter_macho(macho_data)?.enumerate() {
+            let scope_main = SettingsScope::Main;
             let scope_index = SettingsScope::MultiArchIndex(index);
             let scope_arch = SettingsScope::MultiArchCpuType(macho.header.cputype());
 
+            // Older operating system versions don't have support for SHA-256 in
+            // signatures. If the minimum version targeting in the binary doesn't
+            // support SHA-256, we automatically change the digest targeting settings
+            // so the binary will be signed correctly.
+            if let Some(targeting) = find_macho_targeting(macho_data, &macho)? {
+                let sha256_version = targeting.platform.sha256_digest_support()?;
+
+                if !sha256_version.matches(&targeting.minimum_os_version) {
+                    info!(
+                        "activating SHA-1 digests because minimum OS target {} is not {}",
+                        targeting.minimum_os_version, sha256_version
+                    );
+
+                    // This logic is a bit wonky. We want SHA-1 to be present on all binaries
+                    // within a fat binary. So if we need SHA-1 mode, we set the setting on the
+                    // main scope and then clear any overrides on fat binary scopes so our
+                    // settings are canonical.
+                    self.set_digest_type(DigestType::Sha1);
+                    self.add_extra_digest(scope_main.clone(), DigestType::Sha256);
+                    self.extra_digests.remove(&scope_arch);
+                    self.extra_digests.remove(&scope_index);
+                }
+            }
+
+            // The Mach-O can have embedded Info.plist data. Use it if available and not
+            // already defined in settings.
+            if let Some(info_plist) = macho.embedded_info_plist()? {
+                if self.info_plist_data(&scope_main).is_some()
+                    || self.info_plist_data(&scope_index).is_some()
+                    || self.info_plist_data(&scope_arch).is_some()
+                {
+                    info!("using Info.plist data from settings");
+                } else {
+                    info!("preserving Info.plist data already present in Mach-O");
+                    self.set_info_plist_data(scope_index.clone(), info_plist);
+                }
+            }
+
             if let Some(sig) = macho.code_signature()? {
                 if let Some(cd) = sig.code_directory()? {
-                    if self.binary_identifier(&scope_index).is_some()
+                    if self.binary_identifier(&scope_main).is_some()
+                        || self.binary_identifier(&scope_index).is_some()
                         || self.binary_identifier(&scope_arch).is_some()
                     {
                         info!("using binary identifier from settings");
@@ -517,10 +562,45 @@ impl<'key> SigningSettings<'key> {
                         info!("preserving existing binary identifier in Mach-O");
                         self.set_binary_identifier(scope_index.clone(), cd.ident);
                     }
+
+                    if self.team_id.contains_key(&scope_main)
+                        || self.team_id.contains_key(&scope_index)
+                        || self.team_id.contains_key(&scope_arch)
+                    {
+                        info!("using team ID from settings");
+                    } else if let Some(team_id) = cd.team_name {
+                        info!("preserving team ID in existing Mach-O signature");
+                        self.team_id
+                            .insert(scope_index.clone(), team_id.to_string());
+                    }
+
+                    if self.code_signature_flags(&scope_main).is_some()
+                        || self.code_signature_flags(&scope_index).is_some()
+                        || self.code_signature_flags(&scope_arch).is_some()
+                    {
+                        info!("using code signature flags from settings");
+                    } else if !cd.flags.is_empty() {
+                        info!("preserving code signature flags in existing Mach-O signature");
+                        self.set_code_signature_flags(scope_index.clone(), cd.flags);
+                    }
+
+                    if self.runtime_version(&scope_main).is_some()
+                        || self.runtime_version(&scope_index).is_some()
+                        || self.runtime_version(&scope_arch).is_some()
+                    {
+                        info!("using runtime version from settings");
+                    } else if let Some(version) = cd.runtime {
+                        info!("preserving runtime version in existing Mach-O signature");
+                        self.set_runtime_version(
+                            scope_index.clone(),
+                            parse_version_nibbles(version),
+                        );
+                    }
                 }
 
                 if let Some(entitlements) = sig.entitlements()? {
-                    if self.entitlements_plist(&scope_index).is_some()
+                    if self.entitlements_plist(&scope_main).is_some()
+                        || self.entitlements_plist(&scope_index).is_some()
                         || self.entitlements_plist(&scope_arch).is_some()
                     {
                         info!("using entitlements from settings");
@@ -684,25 +764,6 @@ impl<'key> SigningSettings<'key> {
         new
     }
 
-    /// Obtain the executable segment flags for a given scope.
-    pub fn executable_segment_flags(
-        &self,
-        scope: impl AsRef<SettingsScope>,
-    ) -> Option<ExecutableSegmentFlags> {
-        self.executable_segment_flags.get(scope.as_ref()).copied()
-    }
-
-    /// Set executable segment flags for Mach-O binaries.
-    ///
-    /// The incoming flags will replace any already defined flags for the path.
-    pub fn set_executable_segment_flags(
-        &mut self,
-        scope: SettingsScope,
-        flags: ExecutableSegmentFlags,
-    ) {
-        self.executable_segment_flags.insert(scope, flags);
-    }
-
     /// Obtain the `Info.plist` data registered to a given scope.
     pub fn info_plist_data(&self, scope: impl AsRef<SettingsScope>) -> Option<&[u8]> {
         self.info_plist_data
@@ -903,12 +964,6 @@ impl<'key> SigningSettings<'key> {
                 .into_iter()
                 .filter_map(|(key, value)| key_map(key).map(|key| (key, value)))
                 .collect::<BTreeMap<_, _>>(),
-            executable_segment_flags: self
-                .executable_segment_flags
-                .clone()
-                .into_iter()
-                .filter_map(|(key, value)| key_map(key).map(|key| (key, value)))
-                .collect::<BTreeMap<_, _>>(),
             runtime_version: self
                 .runtime_version
                 .clone()
